[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Portfolio client library for Python

[![API Version](https://img.shields.io/badge/api-v3.3.0-blue)](https://developer.factset.com/api-catalog/portfolio-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Portfolio/1.0.0)](https://pypi.org/project/fds.sdk.Portfolio/v/1.0.0)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Allow clients to fetch ModelAccounts Analytics through APIs.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 3.3.0
- SDK version: 1.0.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Portfolio==1.0.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Portfolio==1.0.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Portfolio
from fds.sdk.Portfolio.api import model_accounts_api
from fds.sdk.Portfolio.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Portfolio.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Portfolio.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Portfolio.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = model_accounts_api.ModelAccountsApi(api_client)
    name = "Client:/Folder1/AccountName.Acct" # str | The path and filename of the model account to create or update
    model_account_fields_root = ModelAccountFieldsRoot(
        data=ModelAccountFields(
            iterative={
                "key": {
                    "key": {
                        "key": "key_example",
                    },
                },
            },
            non_iterative={
                "key": {
                    "key": "key_example",
                },
            },
            additional_fields=[
                ModelAccountAdditionalField(
                    iteration=True,
                    name="name_example",
                    type="type_example",
                    description="description_example",
                    split_direction="split_direction_example",
                    size=1,
                    code_page_flag="code_page_flag_example",
                ),
            ],
        ),
        meta={
            "key": "key_example",
        },
    ) # ModelAccountFieldsRoot | The object containing the input values (optional)

    try:
        # Create or update an existing model account
        # example passing only required values which don't have defaults set
        # and optional values
        api_instance.create_or_update_model_account(name, model_account_fields_root=model_account_fields_root)

    except fds.sdk.Portfolio.ApiException as e:
        print("Exception when calling ModelAccountsApi->create_or_update_model_account: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Create or update an existing model account
    #     api_response, http_status_code, response_headers = api_instance.create_or_update_model_account_with_http_info(name, model_account_fields_root=model_account_fields_root)

    # except fds.sdk.Portfolio.ApiException as e:
    #     print("Exception when calling ModelAccountsApi->create_or_update_model_account: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Create or update an existing model account
    #     async_result = api_instance.create_or_update_model_account_async(name, model_account_fields_root=model_account_fields_root)
    # except fds.sdk.Portfolio.ApiException as e:
    #     print("Exception when calling ModelAccountsApi->create_or_update_model_account: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Create or update an existing model account
    #     async_result = api_instance.create_or_update_model_account_with_http_info_async(name, model_account_fields_root=model_account_fields_root)
    # except fds.sdk.Portfolio.ApiException as e:
    #     print("Exception when calling ModelAccountsApi->create_or_update_model_account: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Portfolio

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Portfolio.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Portfolio

configuration = fds.sdk.Portfolio.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Portfolio

configuration = fds.sdk.Portfolio.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Portfolio

configuration = fds.sdk.Portfolio.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/analytics/accounts/v3*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*ModelAccountsApi* | [**create_or_update_model_account**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#create_or_update_model_account) | **PUT** /models/{name} | Create or update an existing model account
*ModelAccountsApi* | [**delete_a_model_account**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#delete_a_model_account) | **DELETE** /models/{name} | Delete model account, takes an account path and name and deletes it.
*ModelAccountsApi* | [**delete_model_account_by_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#delete_model_account_by_date) | **DELETE** /models/{name}/dates/{date} | Delete all entries for a date or specific symbol entries for a date from a previously created account.
*ModelAccountsApi* | [**delete_model_account_by_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#delete_model_account_by_symbol) | **DELETE** /models/{name}/symbols/{symbol} | Delete all entries for a symbol or specific date entries for a symbol from a previously created account.
*ModelAccountsApi* | [**get_account**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#get_account) | **GET** /models/{name} | Get account endpoint, takes an account name and returns underlying data
*ModelAccountsApi* | [**get_account_for_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#get_account_for_date) | **GET** /models/{name}/dates/{date} | Get account endpoint, takes an account name, date and returns underlying data for that date
*ModelAccountsApi* | [**get_account_for_date_and_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#get_account_for_date_and_symbol) | **GET** /models/{name}/dates/{date}/symbols/{symbol} | Get account endpoint, takes an account name and returns underlying data
*ModelAccountsApi* | [**get_account_for_symbol**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#get_account_for_symbol) | **GET** /models/{name}/symbols/{symbol} | Get account endpoint, takes an account name, symbol and returns underlying data for that symbol
*ModelAccountsApi* | [**get_account_schema**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountsApi.md#get_account_schema) | **GET** /models/{name}/schema | Get account schema endpoint, takes an account name and returns its schema


## Documentation For Models

 - [ClientErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ClientErrorResponse.md)
 - [DataAndMetaModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/DataAndMetaModel.md)
 - [Error](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/Error.md)
 - [ErrorSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ErrorSource.md)
 - [ModelAccountAdditionalField](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountAdditionalField.md)
 - [ModelAccountFields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountFields.md)
 - [ModelAccountFieldsRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountFieldsRoot.md)
 - [ModelAccountSchemaRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Portfolio/v3/docs/ModelAccountSchemaRoot.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Portfolio.apis and fds.sdk.Portfolio.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Portfolio.api.default_api import DefaultApi`
- `from fds.sdk.Portfolio.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Portfolio
from fds.sdk.Portfolio.apis import *
from fds.sdk.Portfolio.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

